# frozen_string_literal: true

require_relative '../response'

module Hyde
  module DSL
    # Common methods for Probe objects
    module ProbeMethods
      # Get the current request
      # @return [Hyde::Request]
      def request
        @request
      end

      # Stop execution and generate a boilerplate response with the given code
      # @param errorcode [Integer]
      # @param backtrace [Array(String), nil]
      # @raise [UncaughtThrowError] throws :finish to return back to Server
      def die(errorcode, backtrace: nil)
        throw :finish, [errorcode].append(
          *(@properties["handle.#{errorcode}"] or
            @properties["handle.default"]).call(
              errorcode,
              backtrace: backtrace
            )
        )
      end

      # Bounce request to the next handler
      # @raise [UncaughtThrowError] throws :break to get out of the callback
      def bounce
        raise :break
      end

      # Set response status (generate response if one doesn't exist yet)
      # @param status [Integer] http status code
      def status(status)
        @response = (@response or Hyde::Response.new)
        @response.status = status
      end

      alias code status

      # Set response header (generate response if one doesn't exist yet)
      # @param key [String] header name
      # @param value [String] header value
      def header(key, value)
        return status(value) if key.downcase == "status"

        if key.match(/(?:[(),\/:;<=>?@\[\]{}"]|[^ -~])/)
          raise StandardError, "header key has invalid characters"
        end

        if value.match(/[^ -~]/)
          raise StandardError, "value key has invalid characters"
        end

        @origin.response = (@origin.response or Hyde::Response.new)
        key = key.downcase
        @origin.response.add_header(key, value)
      end

      # Delete a header value from the headers hash
      # If no value is provided, deletes all key entries
      # @param key [String] header name
      # @param value [String, nil] header value
      def remove_header(key, value = nil)
        return unless @origin.response

        return if key.downcase == "status"

        if key.match(/(?:[(),\/:;<=>?@\[\]{}"]|[^ -~])/)
          raise StandardError, "header key has invalid characters"
        end

        if value&.match(/[^ -~]/)
          raise StandardError, "value key has invalid characters"
        end

        @origin.response.delete_header(key, value)
      end
    end
  end
end
