# frozen_string_literal: true

require_relative 'pattern_matching'
require_relative 'node'
require_relative 'dsl/path_constructors'
require_relative 'util/lookup'

module Hyde
  # Protected interface that provides DSL context for setup block.
  class PathBinding
    include Hyde::DSL::PathConstructors

    def initialize(path)
      @origin = path
    end
  end

  # Primary building block of request navigation.
  class Path < Hyde::Node
    Binding = Hyde::PathBinding

    # @param path [Object] Object to generate {Hyde::Pattern} from
    # @param parent [Hyde::Node] Parent object to inherit properties to
    # @param setup [#call] Setup block
    def initialize(path, parent:, &setup)
      super(path)
      @children = []
      @properties = Hyde::Util::Lookup.new(parent&.properties)

      binding = Binding.new(self)
      binding.instance_exec(&setup)
    end

    # Method callback on successful request navigation.
    # Finds the next appropriate path to go to.
    # @return [Boolean] true if further navigation will be done
    # @raise [UncaughtThrowError] by default throws :response if no matches found.
    def process(request)
      @children.each do |x|
        if (value = x.go(request))
          return value
        end
      end
      value = index(request)
      return value if value

      _die(404)
    rescue StandardError => e
      _die(500, backtrace: [e.to_s] + e.backtrace)
    end

    attr_reader :children, :properties

    private

    # Try to perform indexing on the path if possible
    # @param request [Hyde::Request]
    # @return [Boolean] true if indexing succeeded
    def index(request)
      return false unless request.path.match?(/^\/?$/)

      @properties["index"].each do |index|
        request.path = index
        @children.each do |x|
          if (value = x.go(request))
            return value
          end
        end
      end
      false
    end

    # Handle an errorcode
    # @param errorcode [Integer]
    # @param backtrace [Array(String), nil]
    # @raise [UncaughtThrowError] throws :finish to stop processing
    def _die(errorcode, backtrace: nil)
      throw :finish, [errorcode].append(
        *(@properties["handle.#{errorcode}"] or
          @properties["handle.default"]).call(
            errorcode,
            backtrace: backtrace
          )
      )
    end
  end
end
