# frozen_string_literal: true

require 'uri'

module Hyde
  # Request wrapper for Rack protocol
  class Request
    # @param env [Array]
    def initialize(env)
      # Should not be used under regular circumstances or depended upon.
      @_original_env = env
      # Rack environment variable bindings. Should be public and readonly.
      @request_method = env["REQUEST_METHOD"]
      @script_name = env["SCRIPT_NAME"]
      @path_info = env["PATH_INFO"]
      @server_name = env["SERVER_NAME"]
      @server_port = env["SERVER_PORT"]
      @server_protocol = env["SERVER_PROTOCOL"]
      @headers = init_headers(env)
      @param = {}
      @splat = []
      # Traversal route. Public, writable and readable.
      @path = env["PATH_INFO"].dup
      # Encapsulates all rack variables. Should not be public.
      @rack = init_rack_vars(env)
      # Internal navigation states
      @states = []
    end

    # Returns request body (if POST data exists)
    def body
      @rack.input&.gets
    end

    # Push current navigation state (path, splat, param) onto state stack
    def push_state
      @states.push([@path, @param.dup, @splat.dup])
    end

    # Load last navigation state (path, splat, param) from state stack
    def pop_state
      @path, @param, @splat = @states.pop
    end

    attr_reader :request_method, :script_name, :path_info, :server_name,
                :server_port, :server_protocol, :headers, :param, :splat
    attr_accessor :path

    private

    # Initialize rack parameters struct
    # @param env [Hash]
    # @return Object
    def init_rack_vars(env)
      rack_vars = env.filter_map do |k, v|
        [k.delete_prefix("rack."), v] if k.start_with? "rack."
      end.to_h
      return if rack_vars.empty?

      rack_vars["multipart"] = init_multipart_vars(env)
      rack_keys = rack_vars.keys
      rack_keys_sym = rack_keys.map(&:to_sym)
      Struct.new(*rack_keys_sym)
            .new(*rack_vars.values_at(*rack_keys))
            .freeze
    end

    # Initialize multipart parameters struct
    # @param env [Hash]
    # @return Object
    def init_multipart_vars(env)
      multipart_vars = env.filter_map do |k, v|
        if k.start_with? "rack.multipart"
          [k.delete_prefix("rack.multipart."), v]
        end
      end.to_h
      return if multipart_vars.empty?

      multipart_keys = multipart_vars.keys
      multipart_keys_sym = multipart_keys.map(&:to_sym)
      Struct.new(*multipart_keys_sym)
            .new(*multipart_vars.values_at(*multipart_keys))
            .freeze
    end

    # Iniitalize headers hash
    # @param env [Hash]
    # @return Hash
    def init_headers(env)
      headers = env.filter_map do |name, value|
        [name.delete_prefix("HTTP_"), value] if name.start_with?("HTTP_")
      end.to_h
      headers.merge!({ "CONTENT_TYPE" => env["CONTENT_TYPE"],
                       "CONTENT_LENGTH" => env["CONTENT_LENGTH"] })
      headers.freeze
    end
  end
end
